library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity top is
	port (
		sw0 : in std_logic;
		sw1 : in std_logic;
		int_req : out std_logic_vector(1 downto 0);
		clk : in std_logic);
end top;

architecture Behavioral of top is
	constant C_DELAY : std_logic_vector(0 to 15):= x"000a";
	signal rm_chat_cnt : std_logic_vector(0 to 15) := x"0000";
	signal rm_enable : std_logic;
	signal d0, d1, d2, d3, d4 : std_logic_vector(1 downto 0);
begin
	rm_enable <= '1' when rm_chat_cnt = C_DELAY else '0';
	generate_chat_process : process(clk)
	begin
		if clk'event and clk = '1' then
			if rm_chat_cnt = C_DELAY then
				rm_chat_cnt <= (others => '0');
			else
				rm_chat_cnt <= rm_chat_cnt + 1;
			end if;

		end if;
	end process generate_chat_process;

	----------------------------------------------------------------------------------
	-- remove_chat_processor : generate clk enable for 3 dff to remove chattering  
	----------------------------------------------------------------------------------
	remove_chat_process : process(clk)
	begin
		if clk'event and clk = '1' then
			if rm_enable = '1' then
				d0 <= sw0 & sw1;
				d1 <= d0;
				d2 <= d1;
				d3 <= d2;
				d4 <= d3;
				int_req <= (d0 or d1 or d2 or d3 or d4);
			end if;
		end if;
	end process remove_chat_process;
end Behavioral;

