library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity uart_rx is
port(
	clk : in std_logic;
	rst : in std_logic;
	rx : in std_logic;
	rx_data : out std_logic_vector(7 downto 0));
end uart_rx;

architecture Behavioral of uart_rx is
	type st_type is (idle, start, data, stop);
	signal st : st_type;
	signal cnt : integer range 0 to 651; 
	signal cnt_send : integer range 0 to 255; 
	signal data_run, tick_run : integer range 0 to 17; 
	signal rxd: std_logic_vector(7 downto 0);
	signal tick : std_logic;
begin

	rx_data <= rxd;

	process(clk)
	begin
		if clk'event and clk ='1' then
			if rst = '0' then
				st <= idle;
			else
				case st is
					when idle =>
						if rx = '0' then
							st <= start;
							tick_run <= 0;
						end if;
					when start =>
						if tick = '1' then
							if tick_run = 7 then
								st <= data;
								tick_run <= 0; 
								data_run <= 0;
							else
								tick_run <= tick_run + 1;
							end if;
						end if;
					when data =>
						if tick = '1' then
							if tick_run = 15 then
								tick_run <= 0;
								rxd <= rx & rxd(7 downto 1);
								if data_run = 7 then
									st <= stop;
								else
									data_run <= data_run + 1;
								end if;
							else
								tick_run <= tick_run + 1;
							end if;
						end if;
					when stop =>
						if tick = '1' then
							if tick_run = 15 then
								tick_run <= 0;
								st <= idle;
							else
								tick_run <= tick_run + 1;
							end if;
						end if;
					when others => null;
				end case;	
			end if;
		end if;
	end process;


	make_tick : process(clk)
	begin
		if clk'event and clk = '1' then
			if cnt = 651 then
				tick <= '1';
				cnt <= 0; 
			else
				tick <= '0';
				cnt <= cnt + 1;
			end if;
		end if;
	end process make_tick;

end Behavioral;

