library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity uart_tx is
port(
	clk : in std_logic;
	rst : in std_logic;
	ready : in std_logic;
	din: in std_logic_vector(7 downto 0);
	tx : out std_logic);
end uart_tx;

architecture Behavioral of uart_tx is
	type st_type is (idle, data);
	signal st : st_type;
	signal cnt : integer range 0 to 651; 
	signal data_run, tick_run : integer range 0 to 17; 
	signal tx_data : std_logic_vector(9 downto 0);
	signal tick : std_logic;
	
begin

	tx <= '1' when st = idle else tx_data(0);

	process(clk)
	begin
		if clk'event and clk ='1' then
			if rst = '0' then
				st <= idle;
			else
				case st is
					when idle =>
						if ready = '1' then
							st <= data;
							tx_data <= '1' & din & '0';
							tick_run <= 0;
						end if;
					when data =>
						if tick = '1' then
							tick_run <= tick_run + 1;
							if tick_run = 15 then
								tx_data <= '0' & tx_data(9 downto 1);
								tick_run <= 0; 
								data_run <= data_run + 1;
								if data_run = 9 then 
									data_run <= 0;
									st <= idle;
								else
									st <= data;
								end if;
							end if;
						end if;
					when others => null;
				end case;	
			end if;
		end if;
	end process;

	make_tick : process(clk)
	begin
		if clk'event and clk = '1' then
			if cnt = 651 then
				tick <= '1';
				cnt <= 0; 
			else
				tick <= '0';
				cnt <= cnt + 1;
			end if;
		end if;
	end process make_tick;

end Behavioral;

